/*
 * Copyright 2000 Perforce Software.  All rights reserved.
 *
 * This file is part of Perforce - the FAST SCM System.
 */

/*
 * ChangesHelper - Utility to facilitate synchronous access to
 * the server to get changelist information.
 *
 * Public methods:
 *
 *	ChangesHelper::ChangesHelper() - constructor takes: a file path, a ClientApi
 *                                   to perform the commands with
 *	ChangesHelper::LatestChangeNumber() - returns the latest changenumber for the
 *                                            path specified.
 *
 */

class ChangesHelper : public ClientUser {

    public:
	
	// If maxResults = 0, it will find all the changes
	//
	ChangesHelper( ClientApi & api, const StrPtr & path, unsigned int maxResults = 0,
		const StrPtr * user = NULL, const StrPtr * client = NULL );
	virtual	~ChangesHelper();

	int HighestChangeNumber();
	int LowestChangeNumber();
	int Contains( unsigned int changeNumber );
	
	//
	// Do not modify the result!
	StrDict * GetResult( unsigned int index );
	unsigned int TotalResults();
	
	int ChangeNumber( unsigned int index );
	
    private:
	
	ClientApi * api;
	unsigned int maxResults;
	StrBuf pathSpec;
	StrBuf client;
	StrBuf user;
	
	unsigned int latestChangeNumber;
	
	StrBufDict ** results;
	unsigned int totalResults;
	
	virtual void		OutputStat( StrDict *varList );
	virtual void		HandleError( Error *e ) {};
	virtual void		Finished() {};
	StrDict **	GetResults();

} ;

